"""
CINEMA 4D MAXON LABS
(c) MAXON Computer GmbH, all rights reserved

Author : Adam Maxime
Email : m_adam@maxon.de

Class that represent a c4d file. Path are stored in unicode.
Output are in str with .encode('utf8')
While Input can be in str or utf8 the class will internally convert everything

:param filePath: the full path of the file to use
:return False if the file doesn't exist, otherwise the instance
Basic Usage:
    f = C4DFile("MyFile.c4d")
    if not f: return
    print f.name
"""
import os
import re
import c4d
from Const import Const


class C4DFile(object):
    def __repr__(self):
        return str(self)

    def __str__(self):
        return self.filePath

    def __init__(self, version):
        self._folderPath = unicode()
        self._fileName = unicode()
        self._fileVersion = 0
        self._convertedPath = unicode()
        self._isConvertedDocument = False
        self._versionToCheck = version

    def _Init(self, doc):
        """
        Initialize our data
        :param filePath: the full path(folder + filename) of a c4d file, filePath is already checked to be valid and a c4d file (see __metaclass__)
        :raise ValueError: if a thread is not a c4d.threading.C4DThread object
        """
        self._folderPath = self._GetUnicodeString(doc.GetDocumentPath())
        self._fileName = self._GetUnicodeString(doc.GetDocumentName())

        self._SetDataFromDocument(doc)
        self._FindConvertedFile()
        self._QuitDocument(doc)

        return self

    @property
    def folder(self):
        # Get the folder of the file
        return self._folderPath.encode('utf8')

    @property
    def name(self):
        # Get the name of the file
        return self._fileName.encode('utf8')

    @property
    def filePath(self):
        # Get the Full File Path (folder + file name)
        return os.path.join(self.folder, self.name)

    @property
    def version(self):
        # Get the version of this c4d file
        return self._fileVersion

    @property
    def hasBeenConverted(self):
        return bool(self._convertedPath)

    @property
    def convertedPath(self):
        # Get the converted path
        return self._convertedPath.encode('utf8')

    @property
    def isConvertedFile(self):
        # Check if the current file is the conversion of another file
        return self._isConvertedDocument

    @property
    def isPreR12(self):
        # Get if the version is before R12 or not
        return bool(self._fileVersion < self._versionToCheck)

    def CheckFileAndLoad(self, filePath):
        """
        Check a File an load it if possible
        :param filePath: path to a file or a preset
        :return: c4d.BaseDocument or False, bool: the basedocument loaded or False, True if it's a plugin
        """

        # Already a document
        if isinstance(filePath, c4d.documents.BaseDocument):
            if not filePath.GetDocumentPath():
                #c4d.gui.MessageDialog("Please save {0} before to import it".format(filePath.GetDocumentName()))
                return False
            return filePath

        if not os.path.exists(filePath) and not filePath.startswith("preset://"):
            return False

        # Check type
        if not isinstance(filePath, str) and not isinstance(filePath, unicode):
            return False

        # Check if it's a c4d file
        if not filePath.endswith(".c4d"):
            return False

        # If we are here that mean we can load the file so everything is fine :)
        if isinstance(filePath, unicode):
            filePath = filePath.encode('utf8')

        doc = c4d.documents.LoadDocument(filePath, c4d.SCENEFILTER_PROGRESSALLOWED)

        if not doc:
            return None

        return doc

    def SetMinimalVersion(self, minimalVersion):
        self._versionToCheck = minimalVersion

    def _SetDataFromDocument(self, doc):
        """
        Set The version and check kif the file come from a conversion
        """
        version = 0.0
        isConvertedDocument = False

        if doc[c4d.DOCUMENT_INFO_FILEVERSION]:
            version = float(doc[c4d.DOCUMENT_INFO_FILEVERSION])
        isConvertedDocument = bool(doc.GetDataInstance().GetContainer(Const.PLUGIN_ID).GetBool(Const.BC_IS_CONVERTED_DOC))

        self._fileVersion = version
        self._isConvertedDocument = isConvertedDocument

    def _RefreshData(self):
        """
        Recheck the file version
        """
        doc = self.CheckFileAndLoad(self.filePath)
        if not doc: return False

        self._SetDataFromDocument(doc)
        self._convertedPath = unicode()
        self._FindConvertedFile()
        self._QuitDocument(doc)
        return True

    def _FindConvertedFile(self):
        """
        Find and set _convertedPath if a converted file for input file is found
        """
        if not self.isPreR12 or self.isConvertedFile:
            return True

        files = os.listdir(self._GetUnicodeString(self.folder))
        for f in files:
            fullPath = os.path.join(self._GetUnicodeString(self.folder), f)

            if fullPath == self._GetUnicodeString(self.filePath):
                continue

            if not os.path.isfile(fullPath):
                continue

            if not fullPath.endswith(".c4d"):
                continue

            match = re.findall('^(.*?)..(\d*).{4}$', fullPath)[0]
            if not match or len(match) != 2:
                continue

            if not match[0] or not match[1]:
                continue

            version = float(match[1])
            if not self._versionToCheck < version:
                continue

            fullPathTrunked = os.path.split(match[0] + ".c4d")
            path = self._GetUnicodeString(fullPathTrunked[0])
            name = self._GetUnicodeString(fullPathTrunked[1])
            if name != self._GetUnicodeString(self.name) or path != self._GetUnicodeString(self.folder):
                continue

            ui = C4DFile(self.version)
            success = ui._Init(self.CheckFileAndLoad(fullPath))
            if not success or ui.isPreR12 or not ui.isConvertedFile:
                print
                continue

            self._convertedPath = fullPath
            break

        return True

    def _LoadDocument(self, flags, insertDoc=False):
        """
        Load a c4d file as a c4d.documents.BaseDocument, This function always return a BaseDocument or raise errors.
        :param flags: the flag for loading the documents
        :return: c4d.documents.BaseDocument
        """

        # prevent a file to be loaded multiple times from the same C4DFile
        for doc in self._IterateAllDocsOpened():
            docPath = doc.GetDocumentPath()
            docName = doc.GetDocumentName()

            if os.path.join(docPath, docName) == self.filePath:
                return doc

        doc = c4d.documents.LoadDocument(self.filePath, flags, None)
        if doc is None:
            #c4d.gui.MessageDialog("Document {0} was not loaded".format(self.filePath))
            return False

        # prevent a file to be loaded multiple times from the same C4DFile
        if insertDoc:
            c4d.documents.InsertBaseDocument(doc)
            c4d.documents.SetActiveDocument(doc)

        return doc

    def _IsDocumentOpen(self, doc):
        # Does the document is open inside Cinema 4D
        for d in self._IterateAllDocsOpened():
            if d == doc:
                return True

        return False

    def _QuitDocument(self, doc):
        """
        Quit a given document if it's not loaded inside c4d
        :param doc: c4d.document.BaseDocument to quit
        :return: True if BaseDocument is free otherwise False
        """
        if not isinstance(doc, c4d.documents.BaseDocument):
            return False

        if not self._IsDocumentOpen(doc):
            c4d.documents.KillDocument(doc)

        return True

    def _GetUnicodeString(self, string):
        """
        Get a unicode representation of a string (it can already be a unicode string)
        :param string: the string to "translate"
        :return: Unicode representation of this string
        """
        if isinstance(string, unicode):
            return string

        if isinstance(string, str):
            return string.decode('utf8')

        return None

    def _AskAndGetNewName(self, defaultPath):
        """
        Ask the user until he enter a valid name or exist
        :param defaultPath: The path to test
        :return: path where file can be written
        """
        finalPath = defaultPath
        alreadyCalled = False
        while True:
            fileExist = os.path.exists(finalPath)
            if not fileExist: break
            elif fileExist and alreadyCalled:
                if c4d.gui.QuestionDialog("Are you sure to overwrite {0}".format(finalPath)):
                    break

            bufferName = os.path.split(finalPath)
            newName = c4d.gui.RenameDialog(bufferName[1])
            if newName is None:
                return False
            alreadyCalled = True

            finalPath = os.path.join(bufferName[0], newName)

        return finalPath

    def _IsWrittableFolder(self, path):
        """
        Check if we get writing permission in a given path
        :param path: the path to check the writing permissions
        :return: True if can be written otherwise False
        """
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except:
                return False

        test_path = os.path.join(path, "test.txt")
        can_write = False

        try:
            with open(test_path, 'w') as f:
                f.write("test")
                can_write = True
            os.remove(test_path)
        except:
            can_write = False

        return can_write

    def ConvertToR20(self, finalR20Path=None, isCommandLine=False):
        """
        Convert a C4DFile to a file that can be loaded into the R20
        :param finalR20Path: the final Path for the converted file
        :return: True if success otherwise False
        """
        # Load a document
        flags = c4d.SCENEFILTER_OBJECTS | c4d.SCENEFILTER_MATERIALS | c4d.SCENEFILTER_PROGRESSALLOWED | c4d.SCENEFILTER_NONEWMARKERS
        doc = self._LoadDocument(flags)
        if not doc:
            return False

        # Get the final path
        if finalR20Path is None:
            finalR20Path = self.filePath[:-4] + "_r{0}.c4d".format(Const.CURRENT_C4D_VERSION)

        # If it's a preset we ask were we want to save
        isPreset = False
        if self.filePath.startswith("preset://"):
            isPreset = True
            finalR20Path = c4d.storage.SaveDialog(c4d.FILESELECTTYPE_SCENES, def_file=self.name)

        # Check if we got write access
        finalR20Folder = os.path.split(finalR20Path)[0]
        if not self._IsWrittableFolder(finalR20Folder):
            if not isCommandLine:
                c4d.gui.MessageDialog("Folder {0} don't have right permission".format(finalR20Folder))
            return False

        # Check if there is already an existing file
        if not isPreset:
            finalR20Path = self._AskAndGetNewName(finalR20Path)
            if not finalR20Path:
                return False


        # Mark the document as converted
        bc = c4d.BaseContainer()
        bc.SetBool(Const.BC_IS_CONVERTED_DOC, True)
        doc.GetDataInstance().SetContainer(Const.PLUGIN_ID, bc)

        # Save the document
        if not isCommandLine:
            c4d.StatusSetSpin()
            c4d.StatusSetText("Saving: {0}".format(finalR20Path))

        saveState = False
        if self.filePath.startswith("preset://"):
            saveState = c4d.documents.SaveProject(doc,
                                                                                        c4d.SAVEPROJECT_ASSETS | c4d.SAVEPROJECT_SCENEFILE | c4d.SAVEPROJECT_SHOWMISSINGASSETDIALOG | c4d.SAVEPROJECT_PROGRESSALLOWED | c4d.SAVEPROJECT_DONTFAILONMISSINGASSETS,
                                                                                        finalR20Path[:-4])
            fileName = os.path.split(finalR20Path)[1]
            finalR20Path = os.path.join(finalR20Folder, fileName[:-4], fileName)
        else:
            saveState = c4d.documents.SaveDocument(doc,
                                                                                         finalR20Path,
                                                                                         c4d.SAVEDOCUMENTFLAGS_DONTADDTORECENTLIST,
                                                                                         c4d.FORMAT_C4DEXPORT)
        if not isCommandLine:
            c4d.StatusClear()

        if not saveState:
            if not isCommandLine:
                c4d.gui.MessageDialog("Unable to save: {0}".format(finalR20Path))
            self._QuitDocument(doc)
            return False

        # Quit
        self._QuitDocument(doc)
        self._convertedPath = self._GetUnicodeString(finalR20Path)

        return True

    def _IterateAllDocsOpened(self):
        """
        Generator which iterate all opened c4d.documents.BaseDocument
        """
        doc = c4d.documents.GetFirstDocument()

        while doc:
            yield doc
            doc = doc.GetNext()