"""
CINEMA 4D MAXON LABS
(c) MAXON Computer GmbH, all rights reserved

Author : Adam Maxime
Email : m_adam@maxon.de

Class that handle all interactions of CommandLine behavior
:param cl: str of command line arguments
"""
import json
import os
import c4d

from Const import Const
from Dialog import Dialog

class CommandLine(object):

    def __init__(self, cl):
        self.outFile = str()
        self.processPath = str()
        self.mode = Const.MODE_CHECK
        self.importMode = Const.MODE_FOLDER
        self.version = 12

        self.overwriteOutput = False
        self.json = False

        self.SetProcessPath(cl)
        self.SetOutput(cl)
        self.SetVersion(cl)

        self.SetMode(cl)
        self.SetImportMode(cl)
        self.SetJson(cl)
        self.SetOverwriteOutput(cl)

        result = self.Proccess()
        if not result: return

        if self.mode == Const.MODE_CHECK:
            if not self.OutputData(result): return

        elif self.mode == Const.MODE_CONVERT:
            result = self.Convert(result)
            self.OutputData(result)
            return

        return

    def Convert(self, data):
        """
        Convert input data
        :param data: list of uiFiles
        :return: list of dict see GetResultDictFromUiFIle
        """
        result = list()
        for ui in data:
            ui.ConvertToR20(isCommandLine=True)
            result.append(self.GetResultDictFromUiFIle(ui))

        return result

    def OutputData(self, data):
        """
        Output Data according the mode
        :param data: list returned by Proccess
        :return: True if output is done, otherwise False
        """
        if not self.outFile:
            return self._PrintData(data)

        if not self._IsWrittableFolder(os.path.dirname(self.outFile)):
            return self._PrintData(data)

        if os.path.exists(self.outFile) and not self.overwriteOutput:
            return self._PrintData(data)

        if not self._WriteData(data):
            return self._PrintData(data)

        return True

    def _PrintData(self, data):
        """
        Print the data to the console
        :param data:  list returned by Proccess
        :return: True if output is done, otherwise False
        """
        if self.json:
            print json.dumps(data, indent=4)
        else:
            for ui in data:
                print "{0}, {1}, {2}\n".format(ui["path"], ui["convertedPath"], ui["version"])

        return True

    def _WriteData(self, data):
        """
        Write the data into a file
        :param data:  list returned by Proccess
        :return: True if output is done, otherwise False
        """
        try:
            writeFlag = "w" if self.overwriteOutput else "a"
            with open(self.outFile, writeFlag) as f:
                if self.json:
                    f.write(json.dumps(data, indent=4))
                else:
                    for ui in data:
                        f.write("{0},{1},{2}\n".format(ui["path"], ui["convertedPath"], ui["version"]))

            return True

        except:
            return False

    def _IsWrittableFolder(self, path):
        """
        Check if we get writing permission in a given path
        :param path: the path to check the writing permissions
        :return: True if can be written otherwise False
        """
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except:
                return False

        test_path = os.path.join(path, "test.txt")
        can_write = False

        try:
            with open(test_path, 'w') as f:
                f.write("test")
                can_write = True
            os.remove(test_path)
        except:
            can_write = False

        return can_write

    def Proccess(self):
        """
        According the mode set before, proccess input data and return a list with the result
        :return: list of dict from GetResultDictFromUiFIle
        """
        dlg = Dialog(self.version)

        if not self.processPath:
            return

        dlg._AddPath(self.processPath.rstrip(), self.importMode, True)
        result = list()
        for ui in dlg._listOfUiFile:
            if self.mode == Const.MODE_CHECK:
                result.append(self.GetResultDictFromUiFIle(ui))
            else:
                result.append(ui)

        return result

    def GetResultDictFromUiFIle(self, uiFile):
        """
        Get state of an UiFile in a dict
        :param uiFile: the uiFile to output
        :return: dict{"path": path of the file, "convertedPath": file has been converted, "version": file version}
        """
        bufferDict = dict()
        bufferDict["path"] = uiFile.filePath
        bufferDict["convertedPath"] = uiFile.convertedPath if uiFile.hasBeenConverted else False
        bufferDict["version"] = uiFile.version
        return bufferDict

    def SetJson(self, cl):
        cl = cl.replace(self.processPath, "").replace(self.outFile, "")
        self.json = bool("-Json" in cl)

    def SetOverwriteOutput(self, cl):
        cl = cl.replace(self.processPath, "").replace(self.outFile, "")
        self.overwriteOutput = bool("-OverwriteOutput" in cl)

    def SetMode(self, cl):
        cl = cl.replace(self.processPath, "").replace(self.outFile, "")
        if "-CheckConversion" in cl:
            self.mode = Const.MODE_CHECK
        elif "-Convert" in cl:
            self.mode	= Const.MODE_CONVERT
        else:
            self.mode = Const.MODE_CHECK

    def SetImportMode(self, cl):
        cl = cl.replace(self.processPath, "").replace(self.outFile, "")
        if "-Folder" in cl:
            self.importMode = Const.MODE_FOLDER
        elif "-SubFolder" in cl:
            self.importMode	= Const.MODE_SUBFOLDER
        elif "-OneLevel" in cl:
            self.importMode = Const.MODE_ONEFOLDER
        else:
            self.importMode = Const.MODE_FOLDER

    def SetVersion(self, cl):
        if "-Version" in cl:
            clSplitted = cl.split("-Version")
            if len(clSplitted) > 2:
                return

            path = clSplitted[1].split("'")
            if len(path) < 2:
                return
            try:
                self.version = float(path[1::2][0])
            except:
                self.version = 12

    def SetOutput(self, cl):
        if "-Output" in cl:
            clSplitted = cl.split("-Output")
            if len(clSplitted) > 2:
                return

            path = clSplitted[1].split("'")
            if len(path) < 2:
                return
            self.outFile = path[1::2][0]

    def SetProcessPath(self, cl):
        clSplitted = cl.split("-Output")
        if len(clSplitted) >= 1:
            clSplitted = clSplitted[0]

        clPath = clSplitted.split("'")
        if len(clPath) < 2:
            return False

        self.processPath = clPath[1::2][0]