"""
CINEMA 4D MAXON LABS
(c) MAXON Computer GmbH, all rights reserved

Author : Adam Maxime
Email : m_adam@maxon.de

Class to handle the main dialog
"""

import c4d
import time
import os
import ctypes

from UiFile import UiFile
from Const import Const

class EscThread(c4d.threading.C4DThread):
    def __init__(self):
        self.escPressed = False

    def Main(self):
        thread = c4d.threading.GeGetEscTestThread()
        if not thread:
            return

        while 1:
            if self.TestBreak():
                break

            if thread.TestBreak():
                self.escPressed = True
                break

            time.sleep(1 / 25.)

class Dialog(c4d.gui.GeDialog):

    def __init__(self, version=12):
        self._overwrite = False
        self._errorAlreadyImported = list()
        self._errorConvertedFiles = list()
        self._listOfUiFile = list()
        self.SetMinimalVersion(version)

    def SetMinimalVersion(self, version):
        self._version = version
        for ui in self._listOfUiFile:
            ui.SetMinimalVersion(version)

    def CreateLayout(self):
        self.SetTitle("Pre R{0} Updater v{1}".format(self._version, str(Const.VERSION)))
        self.GroupBegin(Const.DIALOG_BASE_MAIN_GROUP_UI, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, cols=1)
        self.GroupBorderSpace(10, 10, 10, 5)
        self.GroupSpace(0, 5)

        self.CreateLayoutMenu()
        self.CreateLayoutFilter()
        self.CreateLayoutUiFile()

        self.AddSeparatorH(0)

        self.GroupBegin(50, c4d.BFH_SCALEFIT | c4d.BFV_BOTTOM, cols=3)
        self.CreateLayoutBottomMenu()
        self.AddButton(Const.DIALOG_CONVERT_ALL_SELECTED_DOC, c4d.BFH_SCALE, 0, 0, "Convert")
        self.GroupEnd()


        self.GroupEnd()
        return True

    def CreateLayoutFilter(self):
        """
        Create layout for Hide Filter group
        """
        self.GroupBegin(Const.DIALOG_FILTER_GRP+1, c4d.BFH_SCALEFIT | c4d.BFV_CENTER, cols=2)

        self.GroupBegin(Const.DIALOG_FILTER_GRP, c4d.BFH_SCALEFIT | c4d.BFV_CENTER, cols=4, groupflags=c4d.BFV_CMD_EQUALCOLUMNS)
        self.AddStaticText(Const.DIALOG_FILTER_TEXT, c4d.BFH_LEFT | c4d.BFV_CENTER, name="Display: ")

        self.AddCheckbox(Const.DIALOG_FILTER_NEED_CONV, c4d.BFH_CENTER | c4d.BFV_BOTTOM, 0, 0, name="Pre R{0}".format(self._version))
        self.SetDefaultColor(Const.DIALOG_FILTER_NEED_CONV, c4d.COLOR_TEXT, Const.UI_COLOR_NEED_UPDATE)
        self.SetBool(Const.DIALOG_FILTER_NEED_CONV, True)

        self.AddCheckbox(Const.DIALOG_FILTER_CONVERTED, c4d.BFH_CENTER | c4d.BFV_BOTTOM, 0, 0, name="Pre R{0} Converted".format(self._version))
        self.SetDefaultColor(Const.DIALOG_FILTER_CONVERTED, c4d.COLOR_TEXT, Const.UI_COLOR_ALREADY_CONVERTED)

        self.AddCheckbox(Const.DIALOG_FILTER_NO_CONV, c4d.BFH_CENTER | c4d.BFV_BOTTOM, 0, 0, name="Post R{0}".format(self._version))
        self.SetDefaultColor(Const.DIALOG_FILTER_NO_CONV, c4d.COLOR_TEXT, Const.UI_COLOR_GOOD)

        self.GroupEnd()
        self.AddButton(Const.DIALOG_CLEAR_ALL_DOC, c4d.BFH_RIGHT | c4d.BFV_TOP, 0, 10, "Remove All")
        self.GroupEnd()

        self.AddSeparatorH(0)

    def CreateLayoutUiFile(self, refresh=False):
        """
        Create layout for all UiFile
        :param refresh: True when the layout need to be refresh (layout need to already be created one time at least)
        """
        if refresh:
            self.LayoutFlushGroup(Const.DIALOG_BASE_GROUP_UI)
        else:
            self.ScrollGroupBegin(Const.DIALOG_BASE_GROUP_UI_SCROLL, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, c4d.SCROLLGROUP_HORIZ | c4d.SCROLLGROUP_VERT, inith=50)
            self.GroupBegin(Const.DIALOG_BASE_GROUP_UI, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=1, inith=50)
            self.GroupBorderSpace(0, 0, 0, 10)

        # Sort our list
        self._listOfUiFile.sort(key=lambda x: (x.version, x.filePath)) # Sort all our list by version and path
        pre32 = filter(lambda x: x.isPreR12 and not x.hasBeenConverted, self._listOfUiFile) # Get only Pre32
        converted = filter(lambda x: x.hasBeenConverted, self._listOfUiFile) # Get only converted
        post = filter(lambda x: not x.isPreR12, self._listOfUiFile) # Get only post

        self._listOfUiFile = pre32 + converted + post # Merge everything in display order

        # Create each UiFile line
        for i, ui in enumerate(self._listOfUiFile):
            if not self.GetBool(Const.DIALOG_FILTER_CONVERTED) and ui.hasBeenConverted:
                continue

            if not self.GetBool(Const.DIALOG_FILTER_NO_CONV) and not ui.isPreR12:
                continue

            if not self.GetBool(Const.DIALOG_FILTER_NEED_CONV) and ui.isPreR12 and not ui.hasBeenConverted:
                continue


            currentId = Const.DIALOG_BASE_UI + i
            ui.SetBgColor(i)

            self.AddSubDialog(currentId, c4d.BFH_SCALEFIT | c4d.BFV_TOP, inith=15)
            self.AttachSubDialog(ui, currentId)

        self.GroupEnd()
        self.GroupEnd()
        if refresh:
            self.LayoutChanged(Const.DIALOG_BASE_GROUP_UI)

    def CreateLayoutMenu(self):
        """
        Create all entries inside the top Menu
        """
        self.MenuFlushAll()
        self.GroupBeginInMenuLine()

        # IMPORT MENU
        self.MenuSubBegin("File")
        self.MenuAddString(Const.DIALOG_MENU_FILE, "Open file")
        self.MenuAddString(Const.DIALOG_MENU_FOLDER, "Open folder")
        self.MenuAddString(Const.DIALOG_MENU_ONE_SUBFOLDER, "Open one level of subfolders")
        self.MenuAddString(Const.DIALOG_MENU_ALL_SUBFOLDER, "Open all subfolders")

        self.MenuAddSeparator()
        self.MenuAddString(Const.DIALOG_MENU_ADD_CURRENT_DOC, "Open current Document")
        self.MenuAddString(Const.DIALOG_MENU_ADD_ALL_OPEN_DOCS, "Open all Opened Documents")

        if c4d.plugins.FindPlugin(Const.ID_GET_CONTENT_BROWSER):
            self.MenuAddCommand(Const.ID_GET_CONTENT_BROWSER)

        self.MenuSubEnd()

        self.MenuSubBegin("Edit")
        # SELECT MENU
        self.MenuSubBegin("Select")
        self.MenuAddString(Const.DIALOG_MENU_SELECT_ALL, "All")
        self.MenuAddString(Const.DIALOG_MENU_SELECT_NEED_CONVERTION, "Pre R{0}".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_SELECT_CONVERTED, "Pre R{0} Converted".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_SELECT_NO_NEED_CONVERTION, "Post R{0}".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_SELECT_PRESET, "Presets")
        self.MenuSubEnd()

        # DESELECT MENU
        self.MenuSubBegin("Deselect")
        self.MenuAddString(Const.DIALOG_MENU_DESELECT_ALL, "All")
        self.MenuAddString(Const.DIALOG_MENU_DESELECT_NEED_CONVERTION, "Pre R{0}".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_DESELECT_CONVERTED, "Pre R{0} Converted".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_DESELECT_NO_NEED_CONVERTION, "Post R{0}".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_DESELECT_PRESET, "Presets")
        self.MenuSubEnd()

        # REMOVE
        self.MenuSubBegin("Remove")
        self.MenuAddString(Const.DIALOG_MENU_REMOVE_SELECT, "Selected")
        self.MenuAddString(Const.DIALOG_MENU_REMOVE_NEED_CONVERTION, "Pre R{0}".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_REMOVE_CONVERTED, "Pre R{0} Converted".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_REMOVE_NO_NEED_CONVERTION, "Post R{0}".format(self._version))
        self.MenuAddString(Const.DIALOG_MENU_REMOVE_PRESET, "Presets")
        self.MenuSubEnd()

        self.MenuSubEnd()

        self.MenuFinished()
        self.GroupEnd()

    def CreateLayoutBottomMenu(self, refresh=False, currentIndex=None, totalIndex=None, currentFile=None):
        if refresh:
            self.LayoutFlushGroup(Const.DIALOG_BASE_BOTTOM_GROUP)
        else:
            self.GroupBegin(Const.DIALOG_BASE_BOTTOM_GROUP, c4d.BFH_CENTER | c4d.BFV_CENTER, cols=10)
        self.GroupSpace(0,0)

        toConvert, converted, noConvert = self._GetFilesCount()
        if currentIndex is not None:
            toConvert = currentIndex
            converted = totalIndex

        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name="Total Files: {0} ".format(len(self._listOfUiFile)))
        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name="[")

        self.AddStaticText(Const.DIALOG_BASE_BOTTOM_TEXT_NEED_UPDATE, c4d.BFH_LEFT | c4d.BFV_CENTER, name=str(toConvert))

        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name="/")
        self.AddStaticText(Const.DIALOG_BASE_BOTTOM_TEXT_ALREADY_CONVERTED, c4d.BFH_LEFT | c4d.BFV_CENTER, name=str(converted))

        if currentIndex is None:
            self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name="/")
            self.AddStaticText(Const.DIALOG_BASE_BOTTOM_TEXT_GOOD, c4d.BFH_LEFT | c4d.BFV_CENTER, name=str(noConvert))

        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name="]")

        if currentFile:
            self.AddStaticText(12, c4d.BFH_LEFT | c4d.BFV_CENTER, name=" - Loading: {}".format(currentFile))

        self.GroupEnd()

        if currentIndex is None:
            self.SetDefaultColor(Const.DIALOG_BASE_BOTTOM_TEXT_NEED_UPDATE, c4d.COLOR_TEXT, Const.UI_COLOR_NEED_UPDATE)
            self.SetDefaultColor(Const.DIALOG_BASE_BOTTOM_TEXT_ALREADY_CONVERTED, c4d.COLOR_TEXT, Const.UI_COLOR_ALREADY_CONVERTED)
            self.SetDefaultColor(Const.DIALOG_BASE_BOTTOM_TEXT_GOOD, c4d.COLOR_TEXT, Const.UI_COLOR_GOOD)

        if refresh:
            self.LayoutChanged(Const.DIALOG_BASE_BOTTOM_GROUP)

    def CoreMessage(self, id, msg):
        # Handle Content Browser Message from C++ plugin

        if id == Const.ID_GET_CONTENT_BROWSER:

            doc = c4d.documents.GetActiveDocument()
            if not doc: return True

            bc = doc.GetDataInstance().GetContainerInstance(Const.ID_GET_CONTENT_BROWSER)
            if not bc: return True
            totalCnt = len(bc)
            cnt = 0

            escThread = EscThread()
            escThread.Start(mode=c4d.THREADMODE_ASYNC, priority=c4d.THREADPRIORITY_BELOW)

            for k, presetPath in bc:
                if escThread.escPressed:
                    break

                self.CreateLayoutBottomMenu(True, cnt, totalCnt, presetPath)
                self._CheckAndAddDoc(presetPath)
                cnt += 1

            if escThread.IsRunning():
                escThread.End()

            # Clean and refresh
            doc.GetDataInstance().RemoveData(Const.ID_GET_CONTENT_BROWSER)
            self.RefreshLayout()

        return True

    def Message(self, msg, result):

        # Handle Drag
        if msg.GetId() == c4d.BFM_DRAGRECEIVE:

            # Check current drag operation is over the Main Group
            if self.CheckDropArea(Const.DIALOG_BASE_GROUP_UI_SCROLL, msg, True, True):
                drag = self.GetDragObject(msg)
                objType = int(drag["type"])
                obj = drag["object"]

                # Only accept scene preset or object preset
                if objType != c4d.DRAGTYPE_FILENAME_OTHER and objType != c4d.DRAGTYPE_FILENAME_SCENE:
                    self.SetDragDestination(c4d.MOUSE_FORBIDDEN)
                    return c4d.gui.GeDialog.Message(self, msg, result)

                self.SetDragDestination(c4d.MOUSE_POINT_HAND)

                # When the drag finnish
                if msg.GetInt32(c4d.BFM_DRAG_FINISHED):
                    self._AddPath(obj.decode('utf8'))
                    self.RefreshLayout()

                self.KillEvents()
                return True

        return c4d.gui.GeDialog.Message(self, msg, result)

    def RefreshLayout(self):
        """
        Refresh the whole Layouts
        """
        if len(self._errorAlreadyImported):
            c4d.gui.MessageDialog("Files already in list:\n-{0}".format("\n-".join(self._errorAlreadyImported)))
        self._errorAlreadyImported = list()

        if len(self._errorConvertedFiles):
            c4d.gui.MessageDialog("Converted files were not imported:\n-{0}".format("\n-".join(self._errorConvertedFiles)))
        self._errorConvertedFiles = list()

        self.CreateLayoutUiFile(True)
        self.CreateLayoutBottomMenu(True)

    def Command(self, id, msg):

        # Menu Add current Document
        if id == Const.DIALOG_MENU_ADD_CURRENT_DOC:
            self._CheckAndAddDoc(c4d.documents.GetActiveDocument())
            self.RefreshLayout()
            return True

        # Menu Add all opened Documents
        if id == Const.DIALOG_MENU_ADD_ALL_OPEN_DOCS:
            for doc in self._GetAllDocsOpened():
                self._CheckAndAddDoc(doc)
            self.RefreshLayout()
            return True

        # Import DIALOG_MENU_FILE:
        if id == Const.DIALOG_MENU_FILE:
            self._Import(Const.MODE_FILE)
            return True

        # Import DIALOG_MENU_FOLDER:
        if id == Const.DIALOG_MENU_FOLDER:
            self._Import(Const.MODE_FOLDER)
            return True

        # Import DIALOG_MENU_ONE_SUBFOLDER:
        if id == Const.DIALOG_MENU_ONE_SUBFOLDER:
            self._Import(Const.MODE_ONEFOLDER)
            return True

        # Import DIALOG_MENU_ALL_SUBFOLDER:
        if id == Const.DIALOG_MENU_ALL_SUBFOLDER:
            self._Import(Const.MODE_SUBFOLDER)
            return True

        # Filters
        if Const.DIALOG_FILTER_CONVERTED <= id <= Const.DIALOG_FILTER_NEED_CONV:
            self.RefreshLayout()
            return True

        # Menu Clear All
        if id == Const.DIALOG_CLEAR_ALL_DOC:
            self._listOfUiFile = list()
            self.RefreshLayout()
            return True

        # Convert all selected Documents
        if id == Const.DIALOG_CONVERT_ALL_SELECTED_DOC:
            for ui in self._listOfUiFile:
                if not self.GetBool(Const.DIALOG_FILTER_CONVERTED) and ui.hasBeenConverted:
                    continue

                if not self.GetBool(Const.DIALOG_FILTER_NO_CONV) and not ui.isPreR12:
                    continue

                if not self.GetBool(Const.DIALOG_FILTER_NEED_CONV) and ui.isPreR12 and not ui.hasBeenConverted:
                    continue

                if ui.isSelected:
                    ui.ConvertToR20()

            self.RefreshLayout()
            return True

        # Clear a particular elements
        if id == Const.UI_FILE_CLEAR_BUTTON:
            pathToRemove = msg[Const.PLUGIN_ID]
            for ui in reversed(self._listOfUiFile):
                if ui.filePath == pathToRemove:
                    self._listOfUiFile.remove(ui)
                    break
            self.RefreshLayout()
            return True

        # Refresh the Ui from UiFile
        if id == Const.PLUGIN_ID:
            self.RefreshLayout()
            return True

        # SELECT
        if id == Const.DIALOG_MENU_SELECT_ALL:
            f = lambda x: True
            self.Select(f, True)
            return True

        if id == Const.DIALOG_MENU_SELECT_NEED_CONVERTION:
            if not self._TestShift():
                self.DeselectAll()

            f = lambda x: x.isPreR12 and not x.hasBeenConverted
            self.Select(f, True)
            return True

        if id == Const.DIALOG_MENU_SELECT_CONVERTED:
            if not self._TestShift():
                self.DeselectAll()

            f = lambda x: x.isPreR12 and x.hasBeenConverted
            self.Select(f, True)
            return True

        if id == Const.DIALOG_MENU_SELECT_NO_NEED_CONVERTION:
            if not self._TestShift():
                self.DeselectAll()

            f = lambda x: not x.isPreR12
            self.Select(f, True)
            return True

        if id == Const.DIALOG_MENU_SELECT_PRESET:
            if not self._TestShift():
                self.DeselectAll()

            f = lambda x: x.filePath.startswith("preset://")
            self.Select(f, True)
            return True

        if id == Const.DIALOG_MENU_SELECT_PLUGIN:
            if not self._TestShift():
                self.DeselectAll()

            f = lambda x: x.filePath.endswith(("cob", "cof"))
            self.Select(f, True)
            return True

        # DESELECT
        if id == Const.DIALOG_MENU_DESELECT_ALL:
            self.DeselectAll()
            return True

        if id == Const.DIALOG_MENU_DESELECT_NEED_CONVERTION:
            f = lambda x: x.isPreR12 and not x.hasBeenConverted
            self.Select(f, False)
            return True

        if id == Const.DIALOG_MENU_DESELECT_CONVERTED:
            f = lambda x: x.isPreR12 and x.hasBeenConverted
            self.Select(f, False)
            return True

        if id == Const.DIALOG_MENU_DESELECT_NO_NEED_CONVERTION:
            f = lambda x: not x.isPreR12
            self.Select(f, False)
            return True

        if id == Const.DIALOG_MENU_DESELECT_PRESET:
            f = lambda x: x.filePath.startswith("preset://")
            self.Select(f, False)
            return True

        if id == Const.DIALOG_MENU_DESELECT_PLUGIN:
            f = lambda x: x.filePath.endswith(("cob", "cof"))
            self.Select(f, False)
            return True

        # REMOVE
        if id == Const.DIALOG_MENU_REMOVE_SELECT:
            f = lambda x: x.isPreR12 and not x.hasBeenConverted
            self.Remove(f)
            return True
            return True

        if id == Const.DIALOG_MENU_REMOVE_NEED_CONVERTION:
            f = lambda x: x.isPreR12 and not x.hasBeenConverted
            self.Remove(f)
            return True

        if id == Const.DIALOG_MENU_REMOVE_CONVERTED:
            f = lambda x: x.isPreR12 and x.hasBeenConverted
            self.Remove(f)
            return True

        if id == Const.DIALOG_MENU_REMOVE_NO_NEED_CONVERTION:
            f = lambda x: not x.isPreR12
            self.Remove(f)
            return True

        if id == Const.DIALOG_MENU_REMOVE_PRESET:
            f = lambda x: x.filePath.startswith("preset://")
            self.Remove(f)
            return True

        return True

    def Remove(self, func):
        """
        Remove UiFile that return true to the test done by func
        :param func: A function to be called, which return True if the UiFile should be deleted
        """
        for ui in reversed(self._listOfUiFile):
            if func(ui):
                self._listOfUiFile.remove(ui)

        return

    def DeselectAll(self):
        # Deselect all UiFile
        for ui in self._listOfUiFile:
            ui.isSelected = False

    def Select(self, func, state):
        """
        Select UiFile that return true to the test done by func
        :param func: A function to be called, which return True if the UiFile should be selected
        """
        for ui in self._listOfUiFile:
            if func(ui):
                ui.isSelected = state

        return

    def _TestEsc(self):
        """
        Test if Esc is pressed
        :return: True if esc is pressed otherwise False
        """
        bc = c4d.BaseContainer()
        rs = c4d.gui.GetInputState(c4d.BFM_INPUT_KEYBOARD, c4d.KEY_ESC, bc)
        if rs and bc[c4d.BFM_INPUT_VALUE]:
            return True
        return False

    def _TestShift(self):
        """
        Test if shift is pressed
        :return: True if shift is pressed otherwise False
        """
        bc = c4d.BaseContainer()
        rs = c4d.gui.GetInputState(c4d.BFM_INPUT_KEYBOARD, c4d.KEY_SHIFT, bc)
        if rs and bc[c4d.BFM_INPUT_VALUE]:
            return True
        return False

    def _Import(self, mode):
        """
        Import Files/Path according the mode chosen by the user
        :param mode: See Const MODE_FILE, MODE_FOLDER, MODE_ONEFOLDER, MODE_SUBFOLDER
        """
        path = str()
        if mode == Const.MODE_FILE:
            path = c4d.storage.LoadDialog()
        else:
            path = c4d.storage.LoadDialog(flags=c4d.FILESELECT_DIRECTORY)

        if path is None:
            return True

        path = path.rstrip().decode('utf8')
        self._AddPath(path, mode)
        self.RefreshLayout()
        return True

    def _GetFilesCount(self):
        """
        Get the count of UiFile that need to be converted, are converted, is ok
        :return: int, int, int: PreR12, PreR12 already converted, PostR12
        """
        toConvert = 0
        converted = 0
        noConvert = 0

        for ui in self._listOfUiFile:
            if ui.isPreR12 and not ui.hasBeenConverted:
                toConvert += 1

            elif ui.hasBeenConverted:
                converted += 1

            else:
                noConvert += 1

        return toConvert, converted, noConvert

    def _CheckAndAddDoc(self, doc):
        """
        Add an UiFile to _listOfUiFile with the given doc/path to a c4d file
        You need to Redraw yourself
        :param doc: c4d.documents.BaseDocuments or path to a c4d file
        :return: True if success otherwise False.
        """
        fullPath = None

        # Get path from loaded document
        if not isinstance(doc, c4d.documents.BaseDocument) and not isinstance(doc, str) and not isinstance(doc, unicode):
            raise ValueError("doc must be a BaseDocument or str or unicode")

        ui = UiFile(self._version)
        success = ui.Init(doc)
        if not success:
            return False

        if ui in self._listOfUiFile:
            self._errorAlreadyImported.append(ui.name)
            return False

        if ui.isConvertedFile:
            self._errorConvertedFiles.append(ui.name)
            return False

        self._listOfUiFile.append(ui)
        return True

    def _AddPath(self, path, iterationMode=Const.MODE_FOLDER, isCommandLine=False):
        """
        Add all c4d file from a given path in the UiList
        :param path: The path we looking for c4d file
        :param includeSubFolders: if search should include sub folders
        """
        if not os.path.exists(path):
            return False

        if os.path.isfile(path):
            return self._CheckAndAddDoc(path)

        fileNameList = list(self._IterateFolder(path, iterationMode))
        fileNameList = filter(lambda x: x.endswith(".c4d") and not self._IsHidden(x), fileNameList)

        totalCnt = len(fileNameList)

        escThread = None
        if not isCommandLine:
            escThread = EscThread()
            escThread.Start(mode=c4d.THREADMODE_ASYNC, priority=c4d.THREADPRIORITY_BELOW)

        for cnt, f in enumerate(fileNameList):
            if not isCommandLine:
                if escThread.escPressed:
                    break

                self.CreateLayoutBottomMenu(True, cnt, totalCnt, f.encode('utf8'))
            self._CheckAndAddDoc(f)

        if not isCommandLine:
            if escThread.IsRunning():
                escThread.End()

        return True

    def _IterateFolder(self, path, mode=Const.MODE_FOLDER):
        """
        Generator that return all files from a path, path already exist.
        :param path: BasePath to search files
        :param includeSubFolder: If should also return file inside sub folders
        :return: Generator of full path in str
        """
        path = path.rstrip(os.path.sep)

        if mode == Const.MODE_FOLDER:
            for element in os.listdir(path):
                fullPath = os.path.join(path, element)

                if self._IsHidden(fullPath):
                    continue

                if os.path.isfile(fullPath):
                    yield fullPath

        elif mode == Const.MODE_ONEFOLDER:
            countSeparator = path.count(os.path.sep)
            for folder, _, files in os.walk(path):

                if countSeparator + 1 < folder.count(os.path.sep):
                    continue

                for f in files:
                    returnPath = os.path.join(folder, f)
                    if self._IsHidden(returnPath):
                        continue
                    yield returnPath

        elif mode == Const.MODE_SUBFOLDER:
            for folder, _, files in os.walk(path):

                for f in files:
                    returnPath = os.path.join(folder, f)
                    if self._IsHidden(returnPath):
                        continue
                    yield returnPath

    def _GetAllDocsOpened(self):
        """
        Generator which iterate all opened c4d.documents.BaseDocument
        """
        doc = c4d.documents.GetFirstDocument()

        while doc:
            yield doc
            doc = doc.GetNext()

    def _IsHidden(self, path):
        """
        Check if a path is hidden or not
        :param path: the path to check
        :return: True if the pah is hidden otherwise False
        """
        name = os.path.basename(os.path.abspath(path))
        return name.startswith('.') or self._CheckWindowsHiddenAttribute(path)

    def _CheckWindowsHiddenAttribute(self, path):
        """
        Check if a path is hidden only for Windows
        :param path: the path to check
        :return: True if the pah is hidden otherwise False
        """

        try:
            attrs = ctypes.windll.kernel32.GetFileAttributesW(unicode(path))
            assert attrs != -1
            result = bool(attrs & 2)
        except (AttributeError, AssertionError):
            result = False
        return result