"""
CINEMA 4D MAXON LABS
(c) MAXON Computer GmbH, all rights reserved

Author : Adam Maxime
Email : m_adam@maxon.de

Class that represent a C4DFile inside a GeDialog
:param C4DFile: the C4DFile to represent
"""
import platform
import c4d

from C4DFile import C4DFile
from Const import Const

class UiFile(c4d.gui.SubDialog, C4DFile):

    def __eq__(self, other):
        if isinstance(other, UiFile):
            return self.filePath == other.filePath
        return False

    def __ne__(self, other):
        return not self == other

    def __init__(self, version):
        super(UiFile, self).__init__(version)
        self._bgColor = c4d.Vector()
        self._flagSubgroupClosed = True
        self._quicktab = None
        self._isInit = False
        self._version = version

    def Init(self, filePath):
        doc = self.CheckFileAndLoad(filePath)
        if not doc:
            return False

        self._Init(doc)
        self._isInit = True
        return True

    def _RaiseIfNotInit(self):
        if not self._isInit:
            raise ValueError("UIFile is not initialized")

    @property
    def isSelected(self):
        # Get if a UiFile is selected or not
        self._RaiseIfNotInit()

        return self.GetBool(Const.UI_FILE_CHECKBOX)

    @isSelected.setter
    def isSelected(self, value):
        self._RaiseIfNotInit()

        self.SetBool(Const.UI_FILE_CHECKBOX, value)

    def SetBgColor(self, index):
        """
        Set the backGroundColor according an index
        :param index: the index corresponding of the line the UiFile represent
        """
        self._RaiseIfNotInit()

        # Values are linear here (not higher than 1)
        bgColor = self.GetColorRGB(c4d.COLOR_BG)
        bgColor = c4d.Vector(bgColor["r"], bgColor["g"], bgColor["b"])

        # Put variation according index number
        if int(index) % 2:
            bgColor *= 1.1

        # Remap to 255
        bgColor.x = c4d.utils.RangeMap(bgColor.x, 0, 255, 0, 1, 0.001)
        bgColor.y = c4d.utils.RangeMap(bgColor.y, 0, 255, 0, 1, 0.001)
        bgColor.z = c4d.utils.RangeMap(bgColor.z, 0, 255, 0, 1, 0.001)

        # Make a bit lighter
        bgColor *= 1.1

        self._bgColor = bgColor

    def InitValues(self):
        # Define the text color
        self._RaiseIfNotInit()

        color = Const.UI_COLOR_GOOD

        if self.isPreR12:
            color = Const.UI_COLOR_NEED_UPDATE

        if self.hasBeenConverted:
            color = Const.UI_COLOR_ALREADY_CONVERTED

        self.SetDefaultColor(Const.UI_FILE_CHECKBOX, c4d.COLOR_TEXT, color)

        if self.isPreR12 and not self.hasBeenConverted:
            self.SetBool(Const.UI_FILE_CHECKBOX, True)

        return True

    def CreateLayout(self):
        self._RaiseIfNotInit()

        self.GroupBegin(Const.UI_FILE_GROUP, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, cols=1)
        # Set the BackGround
        self.SetDefaultColor(Const.UI_FILE_GROUP, c4d.COLOR_BG, self._bgColor)
        self.GroupBorderSpace(5, 2, 5, 5)

        self.GroupBegin(Const.UI_FILE_GROUP_HEADER, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, cols=6)
        self.AddCheckbox(Const.UI_FILE_CHECKBOX, c4d.BFH_LEFT | c4d.BFV_MASK,initw=0, inith=0, name="R{0}".format(self.version))
        self.AddButton(Const.UI_FILE_CONVERT_BUTTON, c4d.BFH_LEFT, 0, 10, "Convert")
        self.AddButton(Const.UI_FILE_REFRESH_BUTTON, c4d.BFH_LEFT, 0, 10, "Refresh")
        self.AddButton(Const.UI_FILE_OPEN_BUTTON, c4d.BFH_LEFT, 0, 10, "Open")
        explorer = "Explorer" if platform.system() == "Windows" else "Finder"
        self.AddButton(Const.UI_FILE_EXPLORER_BUTTON, c4d.BFH_LEFT, 0, 10, "View in {}".format(explorer))
        self.AddButton(Const.UI_FILE_CLEAR_BUTTON, c4d.BFH_LEFT, 0, 10, "Remove")
        self.GroupEnd()

        bc = c4d.BaseContainer()
        bc[c4d.QUICKTAB_BAR] = True
        bc[c4d.QUICKTAB_BARSUBGROUP] = self.hasBeenConverted
        bc[c4d.QUICKTAB_BARTITLE] = self.filePath
        bc[c4d.QUICKTAB_BGCOLOR] = self._bgColor

        self._quicktab = self.AddCustomGui(Const.UI_FILE_PATH, c4d.CUSTOMGUI_QUICKTAB, "", c4d.BFH_SCALEFIT, 0, 0, bc)

        if self.hasBeenConverted:
            self.GroupBegin(Const.UI_FILE_GROUP_CONVERTED, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT)
            self.GroupBorderSpace(20, 0, 0, 0)
            self.AddStaticText(Const.UI_FILE_GROUP_FILE_CONVERTED, c4d.BFH_LEFT | c4d.BFV_MASK, name=self.convertedPath)
            self.HideElement(Const.UI_FILE_GROUP_CONVERTED, self._flagSubgroupClosed)
            self._quicktab.Select(0, self._flagSubgroupClosed)
            self.GroupEnd()

        self.GroupEnd()

        return True

    def Command(self, id, msg):

        if id == Const.UI_FILE_PATH:
            self._flagSubgroupClosed = not self._flagSubgroupClosed
            self.HideElement(Const.UI_FILE_GROUP_CONVERTED, self._flagSubgroupClosed)
            self.LayoutChanged(Const.UI_FILE_GROUP)

        if id == Const.UI_FILE_CONVERT_BUTTON:
            if self.ConvertToR20():
                self._FullUiRefresh()

        if id == Const.UI_FILE_REFRESH_BUTTON:
            if self._RefreshData():
                self._FullUiRefresh()
            else:
                self.SendClearMessage()

        # Open the doc
        if id == Const.UI_FILE_EXPLORER_BUTTON:
            c4d.storage.ShowInFinder(self.filePath, False)
            return True

        if id == Const.UI_FILE_OPEN_BUTTON:
            self._LoadDocument(c4d.SCENEFILTER_OBJECTS | c4d.SCENEFILTER_MATERIALS | c4d.SCENEFILTER_PROGRESSALLOWED, True)
            return True

        if id == Const.UI_FILE_CLEAR_BUTTON:
            self.SendClearMessage()
            return True

        return True

    def SendClearMessage(self):
        # Send a message to the host GeDialog to remove this ui
        action = c4d.BaseContainer(c4d.BFM_ACTION)
        action.SetLong(c4d.BFM_ACTION_ID, Const.UI_FILE_CLEAR_BUTTON)
        action[Const.PLUGIN_ID] = self.filePath
        self.SendParentMessage(action)

    def _FullUiRefresh(self):
        # Refresh the UI of all UIFiles (doesn't recheck simply refresh the UI)
        self._RaiseIfNotInit()

        action = c4d.BaseContainer(c4d.BFM_ACTION)
        action.SetLong(c4d.BFM_ACTION_ID, Const.PLUGIN_ID)
        self.SendParentMessage(action)